/*
 *
 *  MULti Core Operating System - MultiLayer Kernel (MULCOS-M)
 *                      for System device of CortexM architecture
 *
 *          Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name   arch_device.h
 *
 *  May/15/2024 Created.
 *
 */


#ifndef ARCH_DEVICE_H
#define ARCH_DEVICE_H

#include "typedef.h"


/*
 *      Define access barrier
 */


/* Access barrier instruction */

#define __DSB()  __asm__ __volatile__ ("    dsb;" : : : )
#define __ISB()  __asm__ __volatile__ ("    isb;" : : : )

/* Get system register */

#define GET_IPSR(para)        __asm__ __volatile__ ("    mrs %0, ipsr;" : "=r"(para) : : )
#define GET_BASEPRI_NS(para)  __asm__ __volatile__ ("    mrs %0, basepri_ns;" : "=r"(para) : : )
#define SET_BASEPRI_NS(para)  __asm__ __volatile__ ("    msr basepri_ns, %0;" : : "r"(para) : )


/*
 *      Define hardware setting
 */


/* SysTick Controller */

struct systick_t {
    volatile uint32_t    CSR;                           /* SysTick Control and Status Register */
    volatile uint32_t    RVR;                           /* SysTick Reload Value Register */
    volatile uint32_t    CVR;                           /* SysTick Current Value Register */
    volatile uint32_t    CALIB;                         /* SysTick Calibration Value Register */
};

#define SYSTICK       (*(struct systick_t *)0xE000E010)   /* SysTick Address */
#if (__ARM_ARCH == 8)
#define NS_SYSTICK    (*(struct systick_t *)0xE002E010)   /* SysTick Address (NS Alias) */
#endif


/* Nested Vectored Interrupt Controller(NVIC) */

#define SYSR_STIR     (*(volatile uint32_t *)0xE000EF00)  /* Soft Trigger Interrupt Register */
#define NVIC_ICTR     (*(volatile uint32_t *)0xE000E004)  /* Interrupt Control Type Register */
#define NVIC_ISER     ( (volatile uint32_t *)0xE000E100)  /* Irq Set Enable Register */
#define NVIC_ICER     ( (volatile uint32_t *)0xE000E180)  /* Irq Clear Enable Register */
#define NVIC_ISPR     ( (volatile uint32_t *)0xE000E200)  /* Irq Set Pending Register */
#define NVIC_ICPR     ( (volatile uint32_t *)0xE000E280)  /* Irq Clear Pending Register */
#define NVIC_IABR     ( (volatile uint32_t *)0xE000E300)  /* Irq Active Bit Register */
#define NVIC_IPR      ( (volatile uint8_t  *)0xE000E400)  /* Irq Priority Register */
#if (__ARM_ARCH == 8)
#define NVIC_ITNS     ( (volatile uint32_t *)0xE000E380)  /* Irq Target Non-secure Register */
                                                          /* NS Alias Register */
#define NS_SYSR_STIR  (*(volatile uint32_t *)0xE002EF00)  /* Soft Trigger Interrupt Register */
#define NS_NVIC_ICTR  (*(volatile uint32_t *)0xE002E004)  /* Interrupt Control Type Register */
#define NS_NVIC_ISER  ( (volatile uint32_t *)0xE002E100)  /* Irq Set Enable Register */
#define NS_NVIC_ICER  ( (volatile uint32_t *)0xE002E180)  /* Irq Clear Enable Register */
#define NS_NVIC_ISPR  ( (volatile uint32_t *)0xE002E200)  /* Irq Set Pending Register */
#define NS_NVIC_ICPR  ( (volatile uint32_t *)0xE002E280)  /* Irq Clear Pending Register */
#define NS_NVIC_IABR  ( (volatile uint32_t *)0xE002E300)  /* Irq Active Bit Register */
#define NS_NVIC_IPR   ( (volatile uint8_t  *)0xE002E400)  /* Irq Priority Register */
#endif

#define SYSR_CPUID    (*(volatile uint32_t *)0xE000ED00)  /* CPUID Base Register */
#define SYSR_ICSR     (*(volatile uint32_t *)0xE000ED04)  /* Interrupt Control Status Register */
#define SYSR_VTOR     (*(volatile uint32_t **)0xE000ED08) /* Vector Table Offset Register */
#define SYSR_AIRCR    (*(volatile uint32_t *)0xE000ED0C)  /* Application Interrupt/Reset Control Register */
#define SYSR_SCR      (*(volatile uint32_t *)0xE000ED10)  /* System Control Register */
#define SYSR_CCR      (*(volatile uint32_t *)0xE000ED14)  /* Configuration Control Register */
#define SYSR_SHPR     ( (volatile uint8_t  *)0xE000ED18)  /* System Handler Priority Register */
#define SYSR_SHCSR    (*(volatile uint32_t *)0xE000ED24)  /* System Handler Control and State Register */
#define SYSR_CFSR     (*(volatile uint32_t *)0xE000ED28)  /* Configurable Fault Status Register */
#define SYSR_MMFSR    (*(volatile uint8_t  *)0xE000ED28)  /* MemManage Fault Status Register */
#define SYSR_BFSR     (*(volatile uint8_t  *)0xE000ED29)  /* BusFault Status Register */
#define SYSR_UFSR     (*(volatile uint16_t *)0xE000ED2A)  /* UsageFault Status Register */
#define SYSR_CPACR    (*(volatile uint32_t *)0xE000ED88)  /* Coprocessor Access Control Register */
#define SYSR_ACTLR    (*(volatile uint32_t *)0xE000E008)  /* Auxiliary Control Register */
#if (__ARM_ARCH == 8)
#define SYSR_NSACR    (*(volatile uint32_t *)0xE000ED8C)  /* Non-secure Access Control Register */
                                                          /* NS Alias Register */
#define NS_SYSR_CPUID (*(volatile uint32_t *)0xE002ED00)  /* CPUID Base Register */
#define NS_SYSR_ICSR  (*(volatile uint32_t *)0xE002ED04)  /* Interrupt Control Status Register */
#define NS_SYSR_VTOR  (*(volatile uint32_t **)0xE002ED08) /* Vector Table Offset Register */
#define NS_SYSR_AIRCR (*(volatile uint32_t *)0xE002ED0C)  /* Application Interrupt/Reset Control Register */
#define NS_SYSR_SCR   (*(volatile uint32_t *)0xE002ED10)  /* System Control Register */
#define NS_SYSR_CCR   (*(volatile uint32_t *)0xE002ED14)  /* Configuration Control Register */
#define NS_SYSR_SHPR  ( (volatile uint8_t  *)0xE002ED18)  /* System Handler Priority Register */
#define NS_SYSR_SHCSR (*(volatile uint32_t *)0xE002ED24)  /* System Handler Control and State Register */
#define NS_SYSR_CFSR  (*(volatile uint32_t *)0xE002ED28)  /* Configurable Fault Status Register */
#define NS_SYSR_MMFSR (*(volatile uint8_t  *)0xE002ED28)  /* MemManage Fault Status Register */
#define NS_SYSR_BFSR  (*(volatile uint8_t  *)0xE002ED29)  /* BusFault Status Register */
#define NS_SYSR_UFSR  (*(volatile uint16_t *)0xE002ED2A)  /* UsageFault Status Register */
#define NS_SYSR_CPACR (*(volatile uint32_t *)0xE002ED88)  /* Coprocessor Access Control Register */
#define NS_SYSR_ACTLR (*(volatile uint32_t *)0xE002E008)  /* Auxiliary Control Register */
#endif


/* Memory Protection Unit (MPUC) */

#if (__ARM_ARCH >= 7)
#define MPU_TYPE      (*(volatile uint32_t *)0xE000ED90)  /* MPU type Register */
#define MPU_CTRL      (*(volatile uint32_t *)0xE000ED94)  /* MPU control Register */
#define MPU_RNR       (*(volatile uint32_t *)0xE000ED98)  /* MPU region number Register */
#if (__ARM_ARCH == 7)
#define MPU_RBAR      (*(volatile uint32_t *)0xE000ED9C)  /* MPU region base address Register */
#define MPU_RASR      (*(volatile uint32_t *)0xE000EDA0)  /* MPU region attribute and size Register */
#define MPU_RBAR_A1   (*(volatile uint32_t *)0xE000EDA4)  /* Alias 1 of RBAR */
#define MPU_RASR_A1   (*(volatile uint32_t *)0xE000EDA8)  /* Alias 1 of RASR */
#define MPU_RBAR_A2   (*(volatile uint32_t *)0xE000EDAC)  /* Alias 2 of RBAR */
#define MPU_RASR_A2   (*(volatile uint32_t *)0xE000EDB0)  /* Alias 2 of RASR */
#define MPU_RBAR_A3   (*(volatile uint32_t *)0xE000EDB4)  /* Alias 3 of RBAR */
#define MPU_RASR_A3   (*(volatile uint32_t *)0xE000EDB8)  /* Alias 3 of RASR */

#define MPU_BIT_PriRW     0x01000000u
#define MPU_BIT_UnPriRW   0x03000000u
#define MPU_BIT_PriRO     0x05000000u
#define MPU_BIT_UnPriRO   0x07000000u
#define MPU_BIT_ExeNever  0x10000000u
#define MPU_BIT_Shared    0x00040000u
#define MPU_ATR_Strongly  0x00000000u
#define MPU_ATR_SharedDev 0x00010000u
#define MPU_ATR_NonShaDev 0x00100000u

#define MPU_ATR_CacheNon  0x00080000u
#define MPU_ATR_CacheWTNA 0x00020000u
#define MPU_ATR_CacheWBNA 0x00030000u
#define MPU_ATR_CacheWBA  0x000B0000u

#define MPU_BIT_Disable   0x00000000u
#define MPU_BIT_Enable    0x00000001u

#define MPU_SIZE_32B      (4  << 1)
#define MPU_SIZE_64B      (5  << 1)
#define MPU_SIZE_128B     (6  << 1)
#define MPU_SIZE_256B     (7  << 1)
#define MPU_SIZE_512B     (8  << 1)
#define MPU_SIZE_1KB      (9  << 1)
#define MPU_SIZE_2KB      (10 << 1)
#define MPU_SIZE_4KB      (11 << 1)
#define MPU_SIZE_8KB      (12 << 1)
#define MPU_SIZE_16KB     (13 << 1)
#define MPU_SIZE_32KB     (14 << 1)
#define MPU_SIZE_64KB     (15 << 1)
#define MPU_SIZE_128KB    (16 << 1)
#define MPU_SIZE_256KB    (17 << 1)
#define MPU_SIZE_512KB    (18 << 1)
#define MPU_SIZE_1MB      (19 << 1)
#define MPU_SIZE_2MB      (20 << 1)
#define MPU_SIZE_4MB      (21 << 1)
#define MPU_SIZE_8MB      (22 << 1)
#define MPU_SIZE_16MB     (23 << 1)
#define MPU_SIZE_32MB     (24 << 1)
#define MPU_SIZE_64MB     (25 << 1)
#define MPU_SIZE_128MB    (26 << 1)
#define MPU_SIZE_256MB    (27 << 1)
#define MPU_SIZE_512MB    (28 << 1)
#define MPU_SIZE_1GB      (29 << 1)
#define MPU_SIZE_2GB      (30 << 1)
#define MPU_SIZE_4GB      (31 << 1)
#define MPU_SUB_REG(n)    ((n) << 8)

typedef struct t_mputbl {
    volatile uint32_t    rbar;
    volatile uint32_t    rasr;
} T_MPUTBL;

#elif (__ARM_ARCH == 8)
#define MPU_RBAR      (*(volatile uint32_t *)0xE000ED9C)  /* MPU region base address Register */
#define MPU_RLAR      (*(volatile uint32_t *)0xE000EDA0)  /* MPU region limit lddress Register */
#define MPU_RBAR_A1   (*(volatile uint32_t *)0xE000EDA4)  /* Alias 1 of RBAR */
#define MPU_RLSR_A1   (*(volatile uint32_t *)0xE000EDA8)  /* Alias 1 of RLSR */
#define MPU_RBAR_A2   (*(volatile uint32_t *)0xE000EDAC)  /* Alias 2 of RBAR */
#define MPU_RLSR_A2   (*(volatile uint32_t *)0xE000EDB0)  /* Alias 2 of RLSR */
#define MPU_RBAR_A3   (*(volatile uint32_t *)0xE000EDB4)  /* Alias 3 of RBAR */
#define MPU_RLSR_A3   (*(volatile uint32_t *)0xE000EDB8)  /* Alias 3 of RLSR */
#define MPU_MAIR0     (*(volatile uint32_t *)0xE000EDC0)  /* MPU Memory Attribute Indirection Register 0 */
#define MPU_MAIR1     (*(volatile uint32_t *)0xE000EDC4)  /* MPU Memory Attribute Indirection Register 1 */
                                                          /* NS Alias Register */
#define NS_MPU_TYPE    (*(volatile uint32_t *)0xE002ED90) /* MPU type Register */
#define NS_MPU_CTRL    (*(volatile uint32_t *)0xE002ED94) /* MPU control Register */
#define NS_MPU_RNR     (*(volatile uint32_t *)0xE002ED98) /* MPU region number Register */
#define NS_MPU_RBAR    (*(volatile uint32_t *)0xE002ED9C) /* MPU region base address Register */
#define NS_MPU_RLAR    (*(volatile uint32_t *)0xE002EDA0) /* MPU region limit lddress Register */
#define NS_MPU_RBAR_A1 (*(volatile uint32_t *)0xE002EDA4) /* Alias 1 of RBAR */
#define NS_MPU_RLAR_A1 (*(volatile uint32_t *)0xE002EDA8) /* Alias 1 of RLAR */
#define NS_MPU_RBAR_A2 (*(volatile uint32_t *)0xE002EDAC) /* Alias 2 of RBAR */
#define NS_MPU_RLAR_A2 (*(volatile uint32_t *)0xE002EDB0) /* Alias 2 of RLAR */
#define NS_MPU_RBAR_A3 (*(volatile uint32_t *)0xE002EDB4) /* Alias 3 of RBAR */
#define NS_MPU_RLAR_A3 (*(volatile uint32_t *)0xE002EDB8) /* Alias 3 of RLAR */
#define NS_MPU_MAIR0   (*(volatile uint32_t *)0xE002EDC0) /* MPU Memory Attribute Indirection Register 0 */
#define NS_MPU_MAIR1   (*(volatile uint32_t *)0xE002EDC4) /* MPU Memory Attribute Indirection Register 1 */

#define MPU_BIT_NonShare  0x00000000u
#define MPU_BIT_OutShare  0x00000010u
#define MPU_BIT_InnShare  0x00000018u
#define MPU_BIT_PriRW     0x00000000u
#define MPU_BIT_UnPriRW   0x00000002u
#define MPU_BIT_PriRO     0x00000004u
#define MPU_BIT_UnPriRO   0x00000006u
#define MPU_BIT_ExeNever  0x00000001u

#define MPU_BIT_Disable   0x00000000u
#define MPU_BIT_Enable    0x00000001u

#define MPU_ATR_nGnRnE    0x00u
#define MPU_ATR_nGnRE     0x04u
#define MPU_ATR_nGRE      0x08u
#define MPU_ATR_GRE       0x0Cu
#define MPU_ATR_OutNon    0x40u
#define MPU_ATR_OutWTAR   0x90u
#define MPU_ATR_OutAWT    0xA0u
#define MPU_ATR_OutAWTAR  0xB0u
#define MPU_ATR_OutWBAR   0xD0u
#define MPU_ATR_OutAWB    0xE0u
#define MPU_ATR_OutAWBAR  0xF0u
#define MPU_ATR_InnNon    0x04u
#define MPU_ATR_InnWTAR   0x09u
#define MPU_ATR_InnAWT    0x0Au
#define MPU_ATR_InnAWTAR  0x0Bu
#define MPU_ATR_InnWBAR   0x0Du
#define MPU_ATR_InnAWB    0x0Eu
#define MPU_ATR_InnAWBAR  0x0Fu

#define MPU_ATTIR(n)      ((n) << 1)

typedef struct t_mputbl {
    volatile uint32_t    rbar;
    volatile uint32_t    rlar;
} T_MPUTBL;

#endif
#endif


/* Security Attribution Unit (SAU) */

#if (__ARM_ARCH == 8)
#define SAU_CTRL      (*(volatile uint32_t *)0xE000EDD0)  /* SAU control Register */
#define SAU_TYPE      (*(volatile uint32_t *)0xE000EDD4)  /* SAU type Register */
#define SAU_RNR       (*(volatile uint32_t *)0xE000EDD8)  /* SAU region number Register */
#define SAU_RBAR      (*(volatile uint32_t *)0xE000EDDC)  /* SAU region base address Register */
#define SAU_RLAR      (*(volatile uint32_t *)0xE000EDE0)  /* SAU region limit address Register */
#define SFSR          (*(volatile uint32_t *)0xE000EDE4)  /* Secure Fault Status Register */
#define SFAR          (*(volatile uint32_t *)0xE000EDE8)  /* Secure Fault Address Register */
#endif

#define SAU_BIT_NotNSC    0x00000000u
#define SAU_BIT_NSC       0x00000002u
#define SAU_BIT_ENABLE    0x00000001u
#define SAU_BIT_DISABLE   0x00000000u


typedef struct t_sautbl {
    volatile uint32_t    start;
    volatile uint32_t    endat;
} T_SAUTBL;



/* Flowting Point Unit (FPU) */

#if (__ARM_FP > 0)
#define VFP_FPCCR     (*(volatile uint32_t *)0xE000EF34)  /* Floating-Point Context Control Register */
#define VFP_FPCAR     (*(volatile uint32_t *)0xE000EF38)  /* Floating-Point Context Address Register */
#define VFP_FPDSCR    (*(volatile uint32_t *)0xE000EF3C)  /* Floating-Point Default Status Control Register */
#if (__ARM_ARCH == 8)
                                                          /* NS Alias Register */
#define NS_VFP_FPCCR  (*(volatile uint32_t *)0xE002EF34)  /* Floating-Point Context Control Register */
#define NS_VFP_FPCAR  (*(volatile uint32_t *)0xE002EF38)  /* Floating-Point Context Address Register */
#define NS_VFP_FPDSCR (*(volatile uint32_t *)0xE002EF3C)  /* Floating-Point Default Status Control Register */

#define FPCCR_LSPENS     0x20000000u
#define FPCCR_TS         0x04000000u
#endif

#define FPCCR_ASPEN      0x80000000u
#define FPCCR_LSPEN      0x40000000u
#define FPCCR_LSPACT     0x00000001u
#endif

#endif
