/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for RA8 SCI
 *
 *           Copyright (c) 2025 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_ra8_sci.h
 *
 *  Feb/02/2025     Created.
 *
 */

#ifndef _DV_RA8_SCI_H_
#define _DV_RA8_SCI_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define SCI0_BASE       0x40358000      /* SCI 0 Base address */
#define SCI1_BASE       0x40358100      /* SCI 1 Base address */
#define SCI2_BASE       0x40358200      /* SCI 2 Base address */
#define SCI3_BASE       0x40358300      /* SCI 3 Base address */
#define SCI4_BASE       0x40358400      /* SCI 4 Base address */
#define SCI9_BASE       0x40358900      /* SCI 9 Base address */

#define NS_BASE_OFFSET  0x10000000      /* NonSecure peripheral offset address */


/* Evento number */

#define EVENT_SCI0_BASE         0x124
#define EVENT_SCI1_BASE         0x12B
#define EVENT_SCI2_BASE         0x132
#define EVENT_SCI3_BASE         0x139
#define EVENT_SCI4_BASE         0x140
#define EVENT_SCI9_BASE         0x163

#define EVENT_RXI_OFFSET        0
#define EVENT_TXI_OFFSET        1
#define EVENT_TEI_OFFSET        2
#define EVENT_ERI_OFFSET        3


/* Registers */

typedef struct {
    union
    {
        UW  RDR;        /* Receive Data register                               */
        UB  RDR_BY;     /* Receive Data register (byte access)                 */
    };
    union
    {
        UW  TDR;        /* Transmit Data register                              */
        UB  TDR_BY;     /* Transmit Data register (byte access)                */
    };
    UW  CCR0;           /* Common Control register 0                           */
    UW  CCR1;           /* Common Control register 1                           */
    UW  CCR2;           /* Common Control register 2                           */
    UW  CCR3;           /* Common Control register 3                           */
    UW  CCR4;           /* Common Control register 4                           */
    UB  CESR;           /* Communication Enable Status register                */
    UB  reserved;
    UH  reserved1;
    UW  ICR;            /* Simple I2C Control register                         */
    UW  FCR;            /* FIFO Control register                               */
    UW  reserved2;
    UW  MCR;            /* Manchester Control register                         */
    UW  DCR;            /* Driver Control register                             */
    UW  XCR0;           /* Simple LIN(SCIX) Control register 0                 */
    UW  XCR1;           /* Simple LIN(SCIX) Control register 1                 */
    UW  XCR2;           /* Simple LIN(SCIX) Control register 2                 */
    UW  reserved3[2];
    UW  CSR;            /* Common Status register                              */
    UW  ISR;            /* Simple I2C Status register                          */
    UW  FRSR;           /* FIFO Receive Status register                        */
    UW  FTSR;           /* FIFO Transmit Status register                       */
    UW  MSR;            /* Manchester Status register                          */
    UW  XSR0;           /* Simple LIN (SCIX) Status register 0                 */
    UW  XSR1;           /* Simple LIN(SCIX) Status register 1                  */
    UW  reserved4;
    UW  CFCLR;          /* Common Flag Clear register                          */
    UW  ICFCLR;         /* Simple I2C Flag Clear register                      */
    UW  FFCLR;          /* FIFO Flag Clear register                            */
    UW  MFCLR;          /* Manchester Flag Clear register                      */
    UW  XFCLR;          /* Simple LIN(SCIX) Flag Clear register                */
} T_RA8_SCI; 


/* Port access structure */

#ifdef __BUILD_SECURE__
#define REG_SCI0    (*(volatile T_RA8_SCI *)SCI0_BASE)
#define REG_SCI1    (*(volatile T_RA8_SCI *)SCI1_BASE)
#define REG_SCI2    (*(volatile T_RA8_SCI *)SCI2_BASE)
#define REG_SCI3    (*(volatile T_RA8_SCI *)SCI3_BASE)
#define REG_SCI4    (*(volatile T_RA8_SCI *)SCI4_BASE)
#define REG_SCI9    (*(volatile T_RA8_SCI *)SCI9_BASE)
#else
#define REG_SCI0    (*(volatile T_RA8_SCI *)(SCI0_BASE | NS_BASE_OFFSET))
#define REG_SCI1    (*(volatile T_RA8_SCI *)(SCI1_BASE | NS_BASE_OFFSET))
#define REG_SCI2    (*(volatile T_RA8_SCI *)(SCI2_BASE | NS_BASE_OFFSET))
#define REG_SCI3    (*(volatile T_RA8_SCI *)(SCI3_BASE | NS_BASE_OFFSET))
#define REG_SCI4    (*(volatile T_RA8_SCI *)(SCI4_BASE | NS_BASE_OFFSET))
#define REG_SCI9    (*(volatile T_RA8_SCI *)(SCI9_BASE | NS_BASE_OFFSET))
#endif


/* Bit definition of register */

                            /* Receive Data Register */
#define SCI_RDR_FER         0x10000000u
#define SCI_RDR_PER         0x08000000u
#define SCI_RDR_ORER        0x01000000u
#define SCI_RDR_FFER        0x00001000u
#define SCI_RDR_FPER        0x00000800u
#define SCI_RDR_DR          0x00000400u
#define SCI_RDR_MPB         0x00000200u

                            /* Transmit Data Register */
#define SCI_TDR_TSYNC       0x00001000u
#define SCI_TDR_MPBT        0x00000200u

                            /* Common Control Register 0 */
#define SCI_CCR0_SSE        0x01000000u
#define SCI_CCR0_TEIE       0x00200000u
#define SCI_CCR0_TIE        0x00100000u
#define SCI_CCR0_RIE        0x00010000u
#define SCI_CCR0_IDSEL      0x00000400u
#define SCI_CCR0_DCME       0x00000200u
#define SCI_CCR0_MPIE       0x00000100u
#define SCI_CCR0_TE         0x00000010u
#define SCI_CCR0_RE         0x00000001u

                            /* Common Control Register 1 */
#define SCI_CCR1_MFM        0x20000000u
#define SCI_CCR1_MFEN       0x10000000u
#define SCI_CCR1_MFCS       0x07000000u
#define SCI_CCR1_SHARPS     0x00100000u
#define SCI_CCR1_SPLP       0x00010000u
#define SCI_CCR1_RINV       0x00002000u
#define SCI_CCR1_TINV       0x00001000u
#define SCI_CCR1_PM         0x00000200u
#define SCI_CCR1_PE         0x00000100u
#define SCI_CCR1_SPB2IO     0x00000020u
#define SCI_CCR1_SPB2DT     0x00000010u
#define SCI_CCR1_CTSPEN     0x00000002u
#define SCI_CCR1_CTSE       0x00000001u
#define SCI_CCR1_MFCS_SH    24

                            /* Common Control Register 2 */
#define SCI_CCR2_MDDR       0xFF000000u
#define SCI_CCR2_CKS        0x00300000u
#define SCI_CCR2_BRME       0x00010000u
#define SCI_CCR2_BRR        0x0000FF00u
#define SCI_CCR2_ABCSE2     0x00000080u
#define SCI_CCR2_ABCSE      0x00000040u
#define SCI_CCR2_ABCS       0x00000020u
#define SCI_CCR2_BGDM       0x00000010u
#define SCI_CCR2_BCP        0x00000007u
#define SCI_CCR2_MDDR_SH    24
#define SCI_CCR2_CKS_SH     20
#define SCI_CCR2_BRR_SH     8
#define SCI_CCR2_BCP_SH     0

                            /* Common Control Register 3 */
#define SCI_CCR3_BLK        0x20000000u
#define SCI_CCR3_GM         0x10000000u
#define SCI_CCR3_ACS0       0x04000000u
#define SCI_CCR3_CKE        0x03000000u
#define SCI_CCR3_DEN        0x00200000u
#define SCI_CCR3_FM         0x00100000u
#define SCI_CCR3_MP         0x00080000u
#define SCI_CCR3_MOD        0x00070000u
#define SCI_CCR3_RXDESEL    0x00008000u
#define SCI_CCR3_STP        0x00004000u
#define SCI_CCR3_SINV       0x00002000u
#define SCI_CCR3_LSBF       0x00001000u
#define SCI_CCR3_CHR        0x00000300u
#define SCI_CCR3_BPEN       0x00000080u
#define SCI_CCR3_CPOL       0x00000002u
#define SCI_CCR3_CPHA       0x00000001u
#define SCI_CCR3_CKE_SH     24
#define SCI_CCR3_MOD_SH     16
#define SCI_CCR3_CHR_SH     8

                            /* Common Control Register 4 */
#define SCI_CCR4_AET        0x80000000u
#define SCI_CCR4_ATT        0x70000000u
#define SCI_CCR4_AJD        0x08000000u
#define SCI_CCR4_AST        0x07000000u
#define SCI_CCR4_SCKSEL     0x00080000u
#define SCI_CCR4_ATEN       0x00020000u
#define SCI_CCR4_ASEN       0x00010000u
#define SCI_CCR4_CMPD       0x000001FFu
#define SCI_CCR4_ATT_SH     28
#define SCI_CCR4_AST_SH     24
#define SCI_CCR4_CMPD_SH    0

                            /* FIFO Control Register */
#define SCI_FCR_RSTRG       0x1F000000u
#define SCI_FCR_RFRST       0x00800000u
#define SCI_FCR_RTRG        0x001F0000u
#define SCI_FCR_TFRST       0x00008000u
#define SCI_FCR_TTRG        0x00001F00u
#define SCI_FCR_DRES        0x00000001u
#define SCI_FCR_RSTRG_SH    24
#define SCI_FCR_RTRG_SH     16
#define SCI_FCR_TTRG_SH     8

                            /* Driver Control Register */
#define SCI_DCR_DENGT       0x001F0000u
#define SCI_DCR_DEAST       0x00001F00u
#define SCI_DCR_DEPOL       0x00000001u
#define SCI_DCR_DENGT_SH    16
#define SCI_DCR_DEAST_SH    8

                            /* Common Status Register */
#define SCI_CSR_RDRF        0x80000000u
#define SCI_CSR_TEND        0x40000000u
#define SCI_CSR_TDRE        0x20000000u
#define SCI_CSR_FER         0x10000000u
#define SCI_CSR_PER         0x08000000u
#define SCI_CSR_MFF         0x04000000u
#define SCI_CSR_ORER        0x01000000u
#define SCI_CSR_DFER        0x00040000u
#define SCI_CSR_DPER        0x00020000u
#define SCI_CSR_DCMF        0x00010000u
#define SCI_CSR_RXDMON      0x00008000u
#define SCI_CSR_ERS         0x00000010u

                            /* FIFO Receive Status Register */
#define SCI_FRSR_FNUM       0x3F000000u
#define SCI_FRSR_PNUM       0x003F0000u
#define SCI_FRSR_R          0x00003F00u
#define SCI_FRSR_DR         0x00000001u
#define SCI_FRSR_FNUM_SH    24
#define SCI_FRSR_PNUM_SH    16
#define SCI_FRSR_R_SH       8

                            /* FIFO Transmit Status Register */
#define SCI_FTSR_T          0x0000003Fu
#define SCI_FTSR_T_SH       0

                            /* Common Flag Clear Register */
#define SCI_CFCLR_RDRFC     0x80000000u
#define SCI_CFCLR_TDREC     0x20000000u
#define SCI_CFCLR_FERC      0x10000000u
#define SCI_CFCLR_PERC      0x08000000u
#define SCI_CFCLR_MFFC      0x04000000u
#define SCI_CFCLR_ORERC     0x01000000u
#define SCI_CFCLR_DFERC     0x00040000u
#define SCI_CFCLR_DPERC     0x00020000u
#define SCI_CFCLR_DCMFC     0x00010000u
#define SCI_CFCLR_ERSC      0x00000010u

                            /* FIFO Flag Clear Register */
#define SCI_FFCLR_DRC       0x00000001u

                            /* Communication Enable Status Register */
#define SCI_CESR_TIST       0x10u
#define SCI_CESR_RIST       0x01u

#ifdef __cplusplus
}
#endif
#endif /* _DV_RA8_SCI_H_ */
