/***********************************************************************
    MULti Core Operating System - MultiLayer kenel (MULCOS-M)

          NonSecure sample program for EK-RA8M1

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   main.c

    Oct/01/2024 Created.

 ***********************************************************************/

#include <string.h>


#include "kernel.h"
#include "arch_device.h"
#include "drv_uart.h"
#include "dv_ra8m1.h"
#include "drv_ra8_icu.h"


/***********************************************************************
    Definitions of external data and function
 ***********************************************************************/

extern T_CSYS pk_csys;

extern void Led1_On(void);
extern void Led1_Off(void);
extern void Led2_On(void);
extern void Led2_Off(void);
extern void Led3_On(void);
extern void Led3_Off(void);
extern void init_mpu(void);
extern void initial_nvec(void);
extern void initial_fpu(void);
extern void start_test(void);


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

#define   CORE_CLOCK    480000000
#define   PCKA_CLOCK    120000000
#define   PCKB_CLOCK     60000000
#define   PCKC_CLOCK     60000000
#define   PCKD_CLOCK    120000000
#define   SYST_EXTNO    15
#define   EVENT_IRQ12   13
#define   EVENT_IRQ13   14

          /* Using UART0 */
#define   ini_sio       ini_uart0
#define   ref_sio       ref_uart0
#define   ctr_sio       ctr_uart0
#define   fls_sio       fls_uart0
#define   putc_sio      putc_uart0
#define   puts_sio      puts_uart0
#define   getc_sio      getc_uart0
#define   gets_sio      gets_uart0


/***********************************************************************
    Definitions of parameter
 ***********************************************************************/

const VB Message1[] = "The non-secure sample program for the MULCOS-MK system on EK-RA8M1 (Cortex-M85).\n";
const VB Message2[] = "Detected a press on the user button 1.\n";
const VB Message3[] = "Detected a press on the user button 2.\n";


/***********************************************************************
    Definitions of prototype
 ***********************************************************************/

void default_handler(void);
void MainTask(VP_INT stacode);
void cortex_cm_systick(VP_INT exinf);
void cyc_handler1(VP_INT exinf);
void cyc_handler2(VP_INT exinf);
void cortex_cm_init_systick(UW interval);
void gpio_s1_int(VP_INT exinf);
void gpio_s2_int(VP_INT exinf);
void user_button_config(void);
void init_gpio(void);

const T_CTSK ctsk_main = {
#ifdef PRIVILEGE
      .tskatr  = TA_HLNG|TA_PRIV|TA_ACT,
#else
      .tskatr  = TA_HLNG|TA_ACT,
#endif
      .exinf   = (VP_INT)0x5A5A5A5A,
      .task    = (FP)MainTask,
      .itskpri = 1,
      .stksz   = 0x2000,
      .stk     = 0,
      .sstksz  = 0,
      .name    = "MainTask"};
const T_DINH dinh_systick = {
      .inhatr = TA_NULL,
      .inthdr = (FP)cortex_cm_systick,
      .imask  = 0x60};
const T_CCYC ccyc_led1 = {
      .cycatr = TA_HLNG,
      .exinf  = (VP_INT)0x12345678,
      .cychdr = (FP)cyc_handler1,
      .cyctim = 1000,
      .cycphs = 0,
      .name   = "Cyclic LED1"};
const T_CCYC ccyc_led2 = {
      .cycatr = TA_HLNG,
      .exinf  = (VP_INT)0x12345678,
      .cychdr = (FP)cyc_handler2,
      .cyctim = 2000,
      .cycphs = 0,
      .name   = "Cyclic LED2"};
const T_CFLG cflg_us = {
      .flgatr  = TA_TFIFO|TA_WMUL,
      .iflgptn = 0,
      .name    = "user switch"};
const T_UART_DEF uart_def = {
      .baoudrate = 115200,
      .parity    = PAR_NONE,
      .len       = BLEN8,
      .sbit      = SBIT1};


/***********************************************************************
    Definitions of private data
 ***********************************************************************/

ID TskID;
ID FlgID;
ID CycID1;
ID CycID2;
ID IsrID1;
ID IsrID2;
INT proc_switch = 0;

T_CISR cisr_s1;
T_CISR cisr_s2;


/***********************************************************************
    Default handler of abnormal interrupt
        for Interrupts generated by undefined interrupt factors
 ***********************************************************************/

void default_handler(void)
{
    for(;;);
}


/*******************************
        Cyclic handler
 *******************************/

void cyc_handler1(__attribute__((unused)) VP_INT exinf)
{
    Led2_Off();
    stp_cyc(CycID1);
}

void cyc_handler2(__attribute__((unused)) VP_INT exinf)
{
    Led3_Off();
    stp_cyc(CycID2);
}

/***********************************************************************
      Systick interrupt handler
 ***********************************************************************/

void cortex_cm_systick(__attribute__((unused)) VP exinfo)
{
    if ((SYSTICK.CSR & 0x00010000) != 0) {
        if (proc_switch == 0) {
            Led1_On();
        } else if (proc_switch == 50) {
            Led1_Off();
        } else {
            ;
        }
        if (++proc_switch == 500) {
            proc_switch = 0;
        }
    }
}


/***********************************************************************
      Initialize systick and define interrupt handler
 ***********************************************************************/

void cortex_cm_init_systick(UW interval)
{
    ER ret;

    SYSTICK.CSR = 0x00000000;           /* Stop systick timer */

    ret = def_inh(SYST_EXTNO, (T_DINH *)&dinh_systick);
    if (ret >= E_OK) {
        SYSTICK.CVR = 0;
        SYSTICK.RVR = interval-1;
        SYSTICK.CSR = 0x00000007;
    }
}


/***********************************************************************
      User button interrupt handler
 ***********************************************************************/

void gpio_s1_int(__attribute__((unused)) VP_INT exinf)
{
    UW ielsr;

    ielsr = REG_ICU.IELSR[cisr_s1.intno - 16];
    if ((ielsr & IELSR_IR) != 0) {
        REG_ICU.IELSR[cisr_s1.intno - 16] = (ielsr & ~IELSR_IR);

        sta_cyc(CycID1);
        iset_flg(FlgID, 0x00000001);
    }
}

void gpio_s2_int(__attribute__((unused)) VP_INT exinf)
{
    UW ielsr;

    ielsr = REG_ICU.IELSR[cisr_s2.intno - 16];
    if ((ielsr & IELSR_IR) != 0) {
        REG_ICU.IELSR[cisr_s2.intno - 16] = (ielsr & ~IELSR_IR);

        sta_cyc(CycID2);
        iset_flg(FlgID, 0x00000002);
    }
}


/***********************************************************************
        BUTTON pin configurations
 ***********************************************************************/

void user_button_config(void)
{
    ER_ID idno;

    idno = allocate_excp_id(EVENT_IRQ13, 0);
    if (idno > 0) {

        cisr_s1.isratr = TA_HLNG;
        cisr_s1.exinf  = (VP_INT)0;
        cisr_s1.intno  = (INTNO)idno;
        cisr_s1.isr    = gpio_s1_int;
        cisr_s1.imask  = 0x80;
        cisr_s1.name   = "USER S1";

        IsrID1 = acre_isr((T_CISR *)&cisr_s1);
        if (IsrID1 > E_OK) {
            ena_int((INTNO)idno);
        }
    }

    idno = allocate_excp_id(EVENT_IRQ12, 0);
    if (idno > 0) {

        cisr_s2.isratr = TA_HLNG;
        cisr_s2.exinf  = (VP_INT)0;
        cisr_s2.intno  = (INTNO)idno;
        cisr_s2.isr    = gpio_s2_int;
        cisr_s2.imask  = 0x80;
        cisr_s2.name   = "USER S2";

        IsrID2 = acre_isr((T_CISR *)&cisr_s2);
        if (IsrID2 > E_OK) {
            ena_int((INTNO)idno);
        }
    }
}


/*******************************
        Main Task
 *******************************/

void MainTask(__attribute__((unused)) VP_INT exinf)
{
    FLGPTN flgptn;
    UINT txcnt;
    ER ret;

    ini_sio((T_UART_DEF *)&uart_def);
    ctr_sio(ENA_TX|ENA_RX);

    txcnt = strlen(Message1);
    puts_sio((VB *)Message1, &txcnt, TMO_FEVR);

    /* Initialize Cortex-M systick */
#ifdef PRIVILEGE
    cortex_cm_init_systick(CORE_CLOCK/1000);
#else
    loc_cpu();  /* to Privilege mode */
    cortex_cm_init_systick(CORE_CLOCK/1000);
    unl_cpu();
#endif

    FlgID = acre_flg((T_CFLG *)&cflg_us);
    if (FlgID > 0) {
        /* Configuration of user button */
        user_button_config();
    }

    CycID1 = acre_cyc((T_CCYC *)&ccyc_led1);
    CycID2 = acre_cyc((T_CCYC *)&ccyc_led2);

    for(;;) {
        ret = wai_flg(FlgID, 0x00000003, (TWF_ORW |TA_CLR), &flgptn);
        if (ret == E_OK) {
            if ((flgptn & 0x00000001) != 0) {
                Led2_On();
                txcnt = strlen(Message2);
                puts_sio((VB *)Message2, &txcnt, TMO_FEVR);
            }
            if ((flgptn & 0x00000002) != 0) {
                Led3_On();
                txcnt = strlen(Message3);
                puts_sio((VB *)Message3, &txcnt, TMO_FEVR);
            }
        } else {
            dly_tsk(10);
        }
    }
}


/***********************************************************************
      Initialize GPIO
 ***********************************************************************/

void init_gpio(void)
{
    /* Unlock PmnPFS operation */
    REG_PWPR_NS = 0x00;
    REG_PWPR_NS = 0x40;

    /* S1 (P009,IRQ13-DS) pin configuration for using on non-secure world */
    REG_PFS0[9].PnmPFS = (0 << 24) | (0 << 16) | (0 << 15) | (1 << 14) | 
                         (0 << 10) | (1 <<  4) | (0 << 2);
    REG_ICU.IRQCR[13] = (0 << 7) | (1 << 4) | (0 << 0);

    /* S2 (P008,IRQ12-DS) pin configuration for using on non-secure world */
    REG_PFS0[8].PnmPFS = (0 << 24) | (0 << 16) | (0 << 15) | (1 << 14) | 
                         (0 << 10) | (1 <<  4) | (0 << 2);
    REG_ICU.IRQCR[12] = (0 << 7) | (1 << 4) | (0 << 0);

    /* Lock PmnPFS operation */
    REG_PWPR_NS = 0x00;
    REG_PWPR_NS = 0x80;
}


/***************************************
        Function of Initializing
 ***************************************/

int main(void)
{
    ER ret;

    /* Initialize NVEC */
    initial_nvec();

    /* Initialize FPU */
    initial_fpu();

    /* Initialize MPU */
    init_mpu();

    /* Initialize GPIO */
    init_gpio();

    /* Initialize MULCOS-M kernel */
    ret = ini_krn(&pk_csys);
    if (ret == E_OK) {
        TskID = 1;
        ret = cre_tsk(TskID, (T_CTSK *)&ctsk_main);
        if (ret == E_OK) {
            /* Start MULCOS-M kernel */
            sta_krn();
        }
    }
    return ret;
}

/* end */
