/* Linker script to configure memory regions.
 * Need modifying for a specific board.
 *   ROM.ORIGIN: starting address of ROM
 *   ROM.LENGTH: length of ROM
 *   RAM.ORIGIN: starting address of RAM bank 0
 *   RAM.LENGTH: length of SRAM
 */
MEMORY
{
  ROM (rx)                  : ORIGIN = 0x02000000, LENGTH = 0x000F8000 /*  992K */
  NSC (rx)                  : ORIGIN = 0x020F8000, LENGTH = 0x00008000 /*   32K */
  RAM (rw)                  : ORIGIN = 0x22000000, LENGTH = 0x00020000 /*  128K */
  USR (rw)                  : ORIGIN = 0x22020000, LENGTH = 0x00020000 /*  128K */
  SYS (rw)                  : ORIGIN = 0x22040000, LENGTH = 0x0001FC00 /*  127K */
  VCT (rw)                  : ORIGIN = 0x2205FC00, LENGTH = 0x00000400 /*    1K */
  DATA_FLASH (rx)           : ORIGIN = 0x27000000, LENGTH = 0x00002000 /*    8K */
  OPTION_SETTING (r)        : ORIGIN = 0x0300A100, LENGTH = 0x00000018
  OPTION_SETTING_SAS (r)    : ORIGIN = 0x0300A134, LENGTH = 0x000000CC
  OPTION_SETTING_S (r)      : ORIGIN = 0x0300A200, LENGTH = 0x00000100
  OPTION_SETTING_DATA_S (r) : ORIGIN = 0x27030080, LENGTH = 0x00000800
}

/*  Linker script to place sections and symbol values.
 *      Reset_Handler : Entry of reset handler
 *
 */
ENTRY(Reset_Handler)

SECTIONS
{
  /* The startup code into "FLASH" Rom type memory */
  .isr_vector :
  {
    __tz_FLASH_S = ORIGIN(ROM);
    . = ALIGN(4);
    KEEP(*(.intvec))    /* Startup code */
    . = ALIGN(4);
  } >ROM = 0xFF

  /* The program code and other data into "FLASH" Rom type memory */
  .text :
  {
    . = ALIGN(4);
    *(.text)            /* .text sections (code) */
    *(.text*)           /* .text* sections (code) */
    *(.glue_7)          /* glue arm to thumb code */
    *(.glue_7t)         /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    _etext = .;         /* define a global symbols at end of code */
  } >ROM = 0xFF

  /* Constant data into "FLASH" Rom type memory */
  .rodata :
  {
    . = ALIGN(4);
    *(.rodata)          /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)         /* .rodata* sections (constants, strings, etc.) */
    . = ALIGN(4);
  } >ROM = 0xFF

  .ARM.extab (READONLY) : 
  {
    . = ALIGN(4);
    *(.ARM.extab* .gnu.linkonce.armextab.*)
    . = ALIGN(4);
  } >ROM = 0xFF

  .ARM (READONLY) : 
  {
    . = ALIGN(4);
    __exidx_start = .;
    *(.ARM.exidx*)
    __exidx_end = .;
    . = ALIGN(4);
  } >ROM = 0xFF

  .preinit_array (READONLY) :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__preinit_array_start = .);
    KEEP (*(.preinit_array*))
    PROVIDE_HIDDEN (__preinit_array_end = .);
    . = ALIGN(4);
  } >ROM = 0xFF

  .init_array (READONLY) :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__init_array_start = .);
    KEEP (*(SORT(.init_array.*)))
    KEEP (*(.init_array*))
    PROVIDE_HIDDEN (__init_array_end = .);
    . = ALIGN(4);
  } >ROM = 0xFF

  .fini_array (READONLY) :
  {
    . = ALIGN(4);
    PROVIDE_HIDDEN (__fini_array_start = .);
    KEEP (*(SORT(.fini_array.*)))
    KEEP (*(.fini_array*))
    PROVIDE_HIDDEN (__fini_array_end = .);
    . = ALIGN(4);
  } >ROM = 0xFF

  .gnu.sgstubs :
  {
    . = ALIGN(32);
    *(.gnu.sgstubs*)   /* Secure Gateway stubs */
    . = ALIGN(32);
  } >NSC = 0xFF

  /* Used by the startup to initialize data */
  _sidata = LOADADDR(.data);

  __tz_FLASH_N = ORIGIN(NSC) + LENGTH(NSC);

  __tz_RAM_S = ORIGIN(RAM);

  /* Initialized data sections into "RAM" Ram type memory */
  .data :
  {
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */
    *(.RamFunc)        /* .RamFunc sections */
    *(.RamFunc*)       /* .RamFunc* sections */

    . = ALIGN(4);
    _edata = .;        /* define a global symbol at data end */

  } >RAM AT> ROM

  /* User_heap_stack section, used to check that there is enough "RAM" Ram  type memory left */
  ._user_heap_stack :
  {
    . = ALIGN(8);
    PROVIDE ( end = . );
    PROVIDE ( _end = . );
    . = ALIGN(8);
  } >RAM

  /* Uninitialized data section into "RAM" Ram type memory */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss section */
    _sbss = .;         /* define a global symbol at bss start */
    __bss_start__ = _sbss;
    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    _ebss = .;         /* define a global symbol at bss end */
    __bss_end__ = _ebss;
  } >RAM

  __tz_RAM_N = __bss_end__;
  __RAM_NSC_START = ALIGN(__bss_end__, 8192);

  /* Data flash. */
  .data_flash :
  {
      . = ORIGIN(DATA_FLASH);
      __tz_DATA_FLASH_S = .;
      __Data_Flash_Start = .;
      KEEP(*(.data_flash*))
      __Data_Flash_End = .;
  } > DATA_FLASH

  __tz_DATA_FLASH_N = __tz_DATA_FLASH_S + LENGTH(DATA_FLASH);

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_S = ORIGIN(OPTION_SETTING);

  .option_setting :
  {
    __OPTION_SETTING_Start = .;
    KEEP(*(.option_setting_ofs0))
    . = __OPTION_SETTING_Start + 0x04;
    KEEP(*(.option_setting_ofs2))
    . = __OPTION_SETTING_Start + 0x10;
    KEEP(*(.option_setting_dualsel))
  } > OPTION_SETTING = 0xFF

  .option_setting :
  {
    KEEP(*(.option_setting_sas))
    __OPTION_SETTING_End = .;
  } > OPTION_SETTING_SAS = 0xFF

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_N = __OPTION_SETTING_Start + 0x80;

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_S_S = ORIGIN(OPTION_SETTING_S);

  .option_setting_s :
  {
    __OPTION_SETTING_S_Start = .;
    KEEP(*(.option_setting_ofs1_sec))
    . = __OPTION_SETTING_S_Start + 0x10;
    KEEP(*(.option_setting_banksel_sec))
    . = __OPTION_SETTING_S_Start + 0x40;
    KEEP(*(.option_setting_bps_sec0))
    . = __OPTION_SETTING_S_Start + 0x44;
    KEEP(*(.option_setting_bps_sec1))
    . = __OPTION_SETTING_S_Start + 0x48;
    KEEP(*(.option_setting_bps_sec2))
    . = __OPTION_SETTING_S_Start + 0x4C;
    KEEP(*(.option_setting_bps_sec3))
    . = __OPTION_SETTING_S_Start + 0x60;
    KEEP(*(.option_setting_pbps_sec0))
    . = __OPTION_SETTING_S_Start + 0x64;
    KEEP(*(.option_setting_pbps_sec1))
    . = __OPTION_SETTING_S_Start + 0x68;
    KEEP(*(.option_setting_pbps_sec2))
    . = __OPTION_SETTING_S_Start + 0x80;
    KEEP(*(.option_setting_ofs1_sel))
    . = __OPTION_SETTING_S_Start + 0x90;
    KEEP(*(.option_setting_banksel_sel))
    . = __OPTION_SETTING_S_Start + 0xC0;
    KEEP(*(.option_setting_bps_sec0_sel))
    . = __OPTION_SETTING_S_Start + 0xC4;
    KEEP(*(.option_setting_bps_sec1_sel))
    . = __OPTION_SETTING_S_Start + 0xC8;
    KEEP(*(.option_setting_bps_sec2_sel))
    . = __OPTION_SETTING_S_Start + 0xCC;
    KEEP(*(.option_setting_bps_sec3_sel))
    __OPTION_SETTING_S_End = .;
  } > OPTION_SETTING_S = 0xFF

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_S_N = __OPTION_SETTING_S_End;

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_DATA_FLASH_S_S = ORIGIN(OPTION_SETTING_DATA_S);

  .option_setting_data_flash_s :
  {
    __OPTION_SETTING_DATA_S_Start = .;
    KEEP(*(.option_setting_fsblctrl0))
    . = __OPTION_SETTING_DATA_S_Start + 0x04;
    KEEP(*(.option_setting_fsblctrl1))
    . = __OPTION_SETTING_DATA_S_Start + 0x08;
    KEEP(*(.option_setting_fsblctrl2))
    . = __OPTION_SETTING_DATA_S_Start + 0x0C;
    KEEP(*(.option_setting_sacc0))
    . = __OPTION_SETTING_DATA_S_Start + 0x10;
    KEEP(*(.option_setting_sacc1))
    . = __OPTION_SETTING_DATA_S_Start + 0x14;
    KEEP(*(.option_setting_samr))
    . = __OPTION_SETTING_DATA_S_Start + 0x2E0;
    __OPTION_SETTING_DATA_S_End = .;
  } > OPTION_SETTING_DATA_S = 0xFF

  /* Symbol required for RA Configuration tool. */
  __tz_OPTION_SETTING_DATA_FLASH_S_N = __OPTION_SETTING_DATA_S_End;

  .system (NOLOAD) :
  {
    __system_mem_top = .;
      *(.sysmem)

    __free_block_btm = ORIGIN(SYS) + LENGTH(SYS) - 0x01C00;
    __idle_lim_sp    = ORIGIN(SYS) + LENGTH(SYS) - 0x01C00;
    __idle_sp        = ORIGIN(SYS) + LENGTH(SYS) - 0x01800;
    __main_lim_sp    = ORIGIN(SYS) + LENGTH(SYS) - 0x01800;
    __main_sp        = ORIGIN(SYS) + LENGTH(SYS) - 0x00000;
  } >SYS

  /* Vector table for RTOS */
  ._vector :
  {
    . = ALIGN(8);
    __vector_table_rtos = .;
  } >VCT

  /* __tz_RAM_N = __main_sp; */

  /* Remove information from the compiler libraries */
  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }

  .ARM.attributes 0 : { *(.ARM.attributes) }
}
